//	Draw4D-Color.vs

uniform vec3	uniScaleFactors;	//	scales position vectors, but not normal vectors
uniform mat4	uniModelViewMatrix,
				uniModelViewProjectionMatrix;
uniform vec3	uniDiffuseEvaluator,
				uniSpecularEvaluator;
uniform vec3	uniColor;			//	(r,g,b) only, no alpha

in vec3			atrPosition;
in vec3			atrNormal;

out vec3		varDiffuseColor;

void main()
{
	vec3	tmpPosition;
	vec4	tmpWorldPosition;
	float	tmpFogFactor,
			tmpDiffuseFactor,
			tmpSpecularFactor;
	vec3	tmpShadedColor,
			tmpHighlightedColor,
			tmpFoggedColor;

	tmpPosition			= atrPosition * uniScaleFactors;
	gl_Position			= uniModelViewProjectionMatrix * vec4(tmpPosition, 1.0);
	
	//	Don't really need full matrix computation, just z-component of the result.
	tmpWorldPosition	= uniModelViewMatrix * vec4(tmpPosition, 1.0);
	tmpFogFactor		= 0.25 * clamp(tmpWorldPosition.z, 0.0, 1.0);
	
	tmpDiffuseFactor	= 0.50 + 0.50 * max(0.0, dot(uniDiffuseEvaluator, atrNormal));
	tmpSpecularFactor	= 0.25 * pow(max(0.0, dot(uniSpecularEvaluator, atrNormal)), 64.0);
// Do I really want specular lighting disabled???
tmpSpecularFactor = 0.0;

	tmpShadedColor		= tmpDiffuseFactor * uniColor;
	tmpHighlightedColor	= tmpShadedColor
						+ vec3(tmpSpecularFactor, tmpSpecularFactor, tmpSpecularFactor);
	tmpFoggedColor		= (1.0 - tmpFogFactor) * tmpHighlightedColor;

	varDiffuseColor		= tmpFoggedColor;
}
