//	Draw4D-HueBlend.fs

//	Technical Note:  Using a texture lookup to convert hue to RGB
//	is faster than using the high-quality algorithmic shader
//	(see "scraps/Draw4D-Content - SLOW VERSION.fs")
//	but slower than the low-quality algorithmic shader
//	(see "scraps/Draw4D-Content - FAST VERSION.fs").
//	For a hypercube on my first-generation iPad,
//	with NUM_REFINEMENTS=4 for the ball, the frame rates are
//
//		~22fps	high-quality algorithmic shader
//		~38fps	this texture-lookup shader
//		~52fps	low-quality algorithmic shader

uniform sampler2D	uniTexture;		//	automatically initialized to 0

uniform float		uniHueKata,		//	= HUE_KATA	(currently 0.00 = red   )
					uniHueAna,		//	= HUE_ANA	(currently 0.75 = purple)
					uniOpacity;		//	∈ [0.0, 1.0]
#ifdef ALLOW_FLASHING
uniform float		uniBrightness;	//	0.0 = black; 1.0 = full color
#endif

in float	varHue,
			varDiffuseFactor,
			varSpecularFactor,
			varFogFactor;	//	0.0 = no fog; 1.0 = full fog

void main()
{
	float	tmpClampedHue;
	vec3	tmpUnshadedColor,
			tmpShadedColor,
			tmpHighlightedColor,
			tmpFoggedColor;

	tmpClampedHue		= clamp(varHue, uniHueKata, uniHueAna);	//	restrict to colors red through purple
	tmpUnshadedColor	= texture(uniTexture, vec2(tmpClampedHue, 0.5)).rgb;
#ifdef ALLOW_FLASHING
	tmpUnshadedColor	= uniBrightness * tmpUnshadedColor;
#endif
	tmpShadedColor		= varDiffuseFactor * tmpUnshadedColor;
	tmpHighlightedColor	= tmpShadedColor
						+ vec3(varSpecularFactor, varSpecularFactor, varSpecularFactor);
	tmpFoggedColor		= (1.0 - varFogFactor) * tmpHighlightedColor;

	gl_FragColor = uniOpacity * vec4(tmpFoggedColor, 1.0);	//	premultiplied (αR,αG,αB,α)
}
